@@ -26,6 +26,7 @@ Single-stream queue-less decodebin
 import gobject
 import gst
 from pitivi.stream import get_pad_id, pad_compatible_stream
+from pitivi.utils import CachedFactoryList
 
 def is_raw(caps):
     """ returns True if the caps are RAW """
@@ -36,6 +37,19 @@ def is_raw(caps):
             return True
     return False
 
+def factoryFilter(factory):
+    if factory.get_rank() < 64 :
+        return False
+
+    klass = factory.get_klass()
+    for cat in ("Demuxer", "Decoder", "Parse"):
+        if cat in klass:
+            return True
+
+    return False
+
+_factoryCache = CachedFactoryList(factoryFilter)
+
 class SingleDecodeBin(gst.Bin):
     """
     A variant of decodebin.
@@ -56,6 +70,7 @@ class SingleDecodeBin(gst.Bin):
                          gst.PAD_SOMETIMES,
                          gst.caps_new_any())
         )
+
     def __init__(self, caps=None, uri=None, stream=None, *args, **kwargs):
         gst.Bin.__init__(self, *args, **kwargs)
         if not caps:
@@ -71,6 +86,9 @@ class SingleDecodeBin(gst.Bin):
             self.log("created urisrc %s / %r" % (self.urisrc.get_name(),
                                                  self.urisrc))
             self.add(self.urisrc)
+            # Set the blocksize to 512kbytes, this will only matter for push-based sources
+            if hasattr(self.urisrc.props, "blocksize"):
+                self.urisrc.props.blocksize = 524288
             self.urisrc.link(self.typefind)
         else:
             self._sinkpad = gst.GhostPad("sink", self.typefind.get_pad("sink"))
@@ -85,8 +103,6 @@ class SingleDecodeBin(gst.Bin):
 
         self._validelements = [] #added elements
 
-        self._factories = self._getSortedFactoryList()
-
         self.debug("stream:%r" % self.stream)
 
         self.pending_newsegment = False
@@ -100,23 +116,6 @@ class SingleDecodeBin(gst.Bin):
         element.connect("pad-added", self._dynamicPadAddedCb)
         element.connect("no-more-pads", self._dynamicNoMorePadsCb)
 
-    def _getSortedFactoryList(self):
-        """
-        Returns the list of demuxers, decoders and parsers available, sorted
-        by rank
-        """
-        def _myfilter(fact):
-            if fact.get_rank() < 64 :
-                return False
-            klass = fact.get_klass()
-            if not ("Demuxer" in klass or "Decoder" in klass or "Parse" in klass):
-                return False
-            return True
-        reg = gst.registry_get_default()
-        res = [x for x in reg.get_feature_list(gst.ElementFactory) if _myfilter(x)]
-        res.sort(lambda a, b: int(b.get_rank() - a.get_rank()))
-        return res
-
     def _findCompatibleFactory(self, caps):
         """
         Returns a list of factories (sorted by rank) which can take caps as
@@ -124,7 +123,7 @@ class SingleDecodeBin(gst.Bin):
         """
         self.debug("caps:%s" % caps.to_string())
         res = []
-        for factory in self._factories:
+        for factory in _factoryCache.get():
             for template in factory.get_static_pad_templates():
                 if template.direction == gst.PAD_SINK:
                     intersect = caps.intersect(template.static_caps.get())
@@ -327,6 +326,10 @@ class SingleDecodeBin(gst.Bin):
         self._unblockPad(target=self._target)
 
     def _padEventCb(self, pad, event):
+        if event.type == gst.EVENT_TAG:
+            self.debug("dropping TAG event")
+            return False
+
         if event.type != gst.EVENT_NEWSEGMENT:
             self.warning("first event: %s is not a NEWSEGMENT, bailing out" %
                     event)
@@ -334,6 +337,7 @@ class SingleDecodeBin(gst.Bin):
             self._unblockPad(target=pad)
             return True
 
+        self.debug("stored pending newsegment")
         self.pending_newsegment = event
         return False
 
@@ -355,7 +359,7 @@ class SingleDecodeBin(gst.Bin):
         Remove unused elements connected to srcpad(s) of element
         """
         self.log("element:%r" % element)
-        for pad in element.src_pads():
+        for pad in list(element.src_pads()):
             if pad.is_linked():
                 peer = pad.get_peer().get_parent()
                 if isinstance(peer, gst.Element):
