@@ -170,9 +170,7 @@ class ElementTreeFormatter(Formatter):
         if filename is not None:
             if isinstance(filename, unicode):
                 filename = filename.encode("utf-8")
-            filename = self.validateSourceURI(filename)
 
-        if filename is not None:
             factory = klass(filename)
         else:
             factory = klass()
@@ -190,6 +188,12 @@ class ElementTreeFormatter(Formatter):
             stream = self._loadStream(stream_element)
             factory.addOutputStream(stream)
 
+        if filename is not None:
+            filename1 = self.validateSourceURI(filename, factory)
+            if filename != filename1:
+                # the file was moved
+                factory.uri = factory.filename = filename1
+
         self._context.factories[element.attrib["id"]] = factory
         return factory
 
@@ -618,6 +622,9 @@ class ElementTreeFormatter(Formatter):
         discoverer.connect("discovery-error", self._discovererDiscoveryErrorCb,
                 project, sources, uris, closure)
 
+        if not sources:
+            self._finishLoadingProject(project)
+            return
         # start the rediscovering from the first source
         source = sources[0]
         discoverer.addUri(source.uri)
