@@ -26,6 +26,7 @@ Widget for the complex view ruler
 import gobject
 import gtk
 import gst
+import cairo
 from pitivi.ui.zoominterface import Zoomable
 from pitivi.log.loggable import Loggable
 from pitivi.utils import time_to_string, Seeker
@@ -46,6 +47,8 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
 
     border = 0
     min_tick_spacing = 3
+    scale = [0, 0, 0, 0.5, 1, 2, 5, 10, 15, 30, 60, 120, 300, 600, 3600]
+    subdivide = ((1, 1.0), (2, 0.5), (10, .25))
 
     def __init__(self, hadj):
         gtk.Layout.__init__(self)
@@ -73,6 +76,9 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         self.max_duration = gst.CLOCK_TIME_NONE
         self.seeker = Seeker(80)
         self.seeker.connect('seek', self._seekerSeekCb)
+        self.min_frame_spacing = 5.0
+        self.frame_height = 5.0
+        self.frame_rate = gst.Fraction(1/1)
 
 ## Zoomable interface override
 
@@ -169,7 +175,7 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
     def _seekerSeekCb(self, seeker, position, format):
         # clamping values within acceptable range
         duration = self.getShadedDuration()
-        if duration == gst.CLOCK_TIME_NONE:
+        if duration in (0, gst.CLOCK_TIME_NONE):
             return
         if position > duration:
             position = duration - (1 * gst.MSECOND)
@@ -214,15 +220,17 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
                                          allocation.height)
             self.pixmap_old_allocated_width = allocation.width
 
-        context = self.pixmap.cairo_create()
-        self.drawBackground(context, allocation)
-        self.drawRuler(context, allocation)
+        self.drawBackground(allocation)
+        self.drawRuler(allocation)
 
-    def draw(self, context):
-        rect = self.get_allocation()
-        self.debug("Ruler draw %s", list(rect))
-        self.drawBackground(context, rect)
-        self.drawRuler(context, rect)
+    def setProjectFrameRate(self, rate):
+        self.frame_rate = rate
+
+        # set the lowest scale based on project framerate
+        self.scale[0] = float(2 / rate)
+        self.scale[1] = float(5 / rate)
+        self.scale[2] = float(10 / rate)
+        self.queue_resize()
 
     def setShadedDuration(self, duration):
         self.info("start/duration changed")
@@ -256,93 +264,100 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
     def getPixelPosition(self):
         return 0
 
-    def drawBackground(self, context, allocation):
-        context.save()
+    def drawBackground(self, allocation):
+        self.pixmap.draw_rectangle(
+            self.style.bg_gc[gtk.STATE_NORMAL],
+            True,
+            0, 0,
+            allocation.width, allocation.height)
 
-        context.set_source_rgb(0.5, 0.5, 0.5)
-        context.rectangle(0, 0, allocation.width, allocation.height)
-        context.fill()
-        context.stroke()
+        offset = int(Zoomable.nsToPixel(self.getShadedDuration())) - self.pixmap_offset
+        if offset > 0:
+            self.pixmap.draw_rectangle(
+                self.style.bg_gc[gtk.STATE_ACTIVE],
+                True,
+                0, 0,
+                offset,
+                allocation.height)
 
-        if self.getShadedDuration() > 0:
-            context.set_source_rgb(0.8, 0.8, 0.8)
-            context.rectangle(0, 0, self.getShadedDurationWidth(), allocation.height)
-            context.fill()
-            context.stroke()
+    def drawRuler(self, allocation):
+        layout = self.create_pango_layout(time_to_string(0))
+        textwidth, textheight = layout.get_pixel_size()
 
-        context.restore()
+        for scale in self.scale:
+            spacing = Zoomable.zoomratio * scale
+            if spacing >= textwidth * 1.5:
+                break
 
-    def drawRuler(self, context, allocation):
-        # there are 4 lengths of tick mark:
-        # full height: largest increments, 1 minute
-        # 3/4 height: 10 seconds
-        # 1/2 height: 1 second
-        # 1/4 height: 1/10 second (might later be changed to 1 frame in
-        #   project framerate)
+        offset = self.pixmap_offset % spacing
 
-        # At the highest level of magnification, all ticmarks are visible. At
-        # the lowest, only the full height tic marks are visible. The
-        # appearance of text is dependent on the spacing between tics: text
-        # only appears when there is enough space between tics for it to be
-        # readable.
-
-        def textSize(text):
-            return context.text_extents(text)[2:4]
-
-        def drawTick(paintpos, height):
-            context.move_to(paintpos, 0)
-            context.line_to(paintpos, allocation.height * height)
-
-        def drawText(paintpos, time, txtwidth, txtheight):
-            # draw the text position
-            time = time_to_string(time)
-            context.move_to( paintpos - txtwidth / 2.0,
-                             allocation.height - 2 )
-            context.show_text( time )
-
-        def drawTicks(interval, height):
-            spacing = zoomRatio * interval
-            offset = self.pixmap_offset % spacing
-            paintpos = float(self.border) + 0.5
-            if offset > 0:
-                paintpos += spacing - offset
-            if spacing >= self.min_tick_spacing:
-                while paintpos < allocation.width:
-                    drawTick(paintpos, height)
-                    paintpos += zoomRatio * interval
-
-        def drawTimes(interval):
-            # figure out what the optimal offset is
-            spacing = zoomRatio * interval
-            offset = self.pixmap_offset % spacing
-            seconds = self.pixelToNs(self.pixmap_offset)
-            paintpos = float(self.border) + 0.5
-            if offset > 0:
-                seconds += self.pixelToNs(spacing - offset)
-                paintpos += spacing - offset
-            textwidth, textheight = textSize(time_to_string(0))
-            if spacing > textwidth:
-                while paintpos < allocation.width:
-                    timevalue = long(seconds)
-                    drawText(paintpos, timevalue, textwidth, textheight)
-                    paintpos += spacing
-                    seconds += long(interval * gst.SECOND)
-
-
-        context.save()
         zoomRatio = self.zoomratio
-        # looks better largest tick doesn't run into the text label
-        interval_sizes = ((60, 0.80), (10, 0.75), (1, 0.5), (0.1, 0.25))
-        for interval, height in interval_sizes:
-            drawTicks(interval, height)
-            drawTimes(interval)
+        self.drawFrameBoundaries(allocation)
+        self.drawTicks(allocation, offset, spacing, scale)
+        self.drawTimes(allocation, offset, spacing, scale, layout)
 
-        #set a slightly thicker line. This forces anti-aliasing, and gives the
-        #a softer appearance
-        context.set_line_width(1.1)
-        context.set_source_rgb(0.4, 0.4, 0.4)
-        context.stroke()
-        context.restore()
+    def drawTick(self, allocation, paintpos, height):
+        paintpos = int(paintpos)
+        height = allocation.height - int(allocation.height * height)
+        self.pixmap.draw_line(
+            self.style.fg_gc[gtk.STATE_NORMAL],
+            paintpos, height, paintpos,
+            allocation.height)
+
+    def drawTicks(self, allocation, offset, spacing, scale):
+        for subdivide, height in self.subdivide:
+            spc = spacing / float(subdivide)
+            dur = scale / float(subdivide)
+            if spc < self.min_tick_spacing:
+                break
+            paintpos = float(self.border) + 0.5
+            if offset > 0:
+                paintpos += spacing - offset
+            while paintpos < allocation.width:
+                self.drawTick(allocation, paintpos, height)
+                paintpos += spc
+
+    def drawTimes(self, allocation, offset, spacing, scale, layout):
+        # figure out what the optimal offset is
+        interval = long(gst.SECOND * scale)
+        seconds = self.pixelToNs(self.pixmap_offset)
+        paintpos = float(self.border) + 2
+        if offset > 0:
+            seconds = seconds - (seconds % interval) + interval
+            paintpos += spacing - offset
+        shaded = self.getShadedDurationWidth()
+
+        while paintpos < allocation.width:
+            timevalue = time_to_string(long(seconds))
+            layout.set_text(timevalue)
+            if paintpos < shaded:
+                state = gtk.STATE_ACTIVE
+            else:
+                state = gtk.STATE_NORMAL
+            self.pixmap.draw_layout(
+                self.style.fg_gc[state],
+                int(paintpos), 0, layout)
+            paintpos += spacing
+            seconds += interval
+
+    def drawFrameBoundaries(self, allocation):
+        ns_per_frame = float(1 / self.frame_rate) * gst.SECOND
+        frame_width = self.nsToPixel(ns_per_frame)
+        if frame_width >= self.min_frame_spacing:
+            offset = self.pixmap_offset % frame_width
+            paintpos = float(self.border) + 0.5
+            height = allocation.height
+            y = int(height - self.frame_height)
+            states = [gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT]
+            paintpos += frame_width - offset
+            frame_num = int(paintpos // frame_width) % 2
+            while paintpos < allocation.width:
+                self.pixmap.draw_rectangle(
+                    self.style.bg_gc[states[frame_num]],
+                    True,
+                    int(paintpos), y, frame_width, height)
+                frame_num = (frame_num + 1) % 2
+                paintpos += frame_width
 
     def drawPosition(self, context, allocation):
         if self.getShadedDuration() <= 0:
