@@ -27,7 +27,6 @@ import os
 import time
 
 from urllib import unquote
-from xml.sax.saxutils import escape
 from gettext import gettext as _
 from gettext import ngettext
 
@@ -40,6 +39,8 @@ from pitivi.stream import VideoStream, AudioStream, TextStream, \
         MultimediaStream
 from pitivi.settings import GlobalSettings
 from pitivi.utils import beautify_length
+from pitivi.ui.common import beautify_factory, factory_name, \
+    beautify_stream
 from pitivi.log.loggable import Loggable
 from pitivi.sourcelist import SourceListError
 
@@ -85,48 +86,6 @@ ui = '''
 INVISIBLE = gtk.gdk.pixbuf_new_from_file(os.path.join(get_pixmap_dir(), 
     "invisible.png"))
 
-def beautify_stream(stream):
-
-    if type(stream) == AudioStream:
-        if stream.raw:
-            templ = ngettext("<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
-                    "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
-                    stream.channels)
-            templ = templ % (stream.channels, stream.rate, stream.width)
-            return templ
-
-        return _("<b>Unknown Audio format:</b> %s") % stream.audiotype
-
-    elif type(stream) == VideoStream:
-        if stream.raw:
-            if stream.framerate.num:
-                templ = _("<b>Video:</b> %d x %d <i>pixels</i> at %.2f<i>fps</i>")
-                templ = templ % (stream.par * stream.width , stream.height,
-                        float(stream.framerate))
-            else:
-                templ = _("<b>Image:</b> %d x %d <i>pixels</i>")
-                templ = templ % (stream.par * stream.width, stream.height)
-            return templ
-        return _("<b>Unknown Video format:</b> %s") % stream.videotype
-
-    elif type(stream) == TextStream:
-        return _("<b>Text:</b> %s") % stream.texttype
-
-    raise NotImplementedError
-
-def beautify_factory(factory):
-    ranks = {VideoStream: 0, AudioStream: 1, TextStream: 2, MultimediaStream: 3}
-    def stream_sort_key(stream):
-        return ranks[type(stream)]
-
-    streams = factory.getOutputStreams()
-    streams.sort(key=stream_sort_key)
-    return ("<b>" + escape(unquote(factory.name)) + "</b>\n" +
-        "\n".join((beautify_stream(stream) for stream in streams)))
-
-def factory_name(factory):
-    return escape(unquote(factory.name))
-
 class SourceList(gtk.VBox, Loggable):
     """ Widget for listing sources """
 
@@ -146,9 +105,6 @@ class SourceList(gtk.VBox, Loggable):
         self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, object, str, str,
             str)
 
-        self.set_border_width(5)
-        self.set_spacing(6)
-
         # Scrolled Window
         self.scrollwin = gtk.ScrolledWindow()
         self.scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
@@ -426,6 +382,7 @@ class SourceList(gtk.VBox, Loggable):
                                                    chooser_action,
                                                    (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE,
                                                     gtk.STOCK_ADD, gtk.RESPONSE_OK))
+        self._importDialog.set_icon_name("pitivi")
         self._importDialog.props.extra_widget = close_after
         self._importDialog.set_default_response(gtk.RESPONSE_OK)
         self._importDialog.set_select_multiple(True)
